// src/installer/downloader.rs
use anyhow::{Result, Context, anyhow};
use reqwest::Client;
use tokio::io::AsyncWriteExt;
use std::path::{Path, PathBuf};
use std::sync::Arc;
use std::time::Duration;
use futures_util::StreamExt;
use tokio::sync::Mutex;
use std::fs::metadata;
use reqwest::header::{HeaderMap, HeaderValue, RANGE, USER_AGENT};
use log::{info, warn};
use crate::app::GameType;

pub struct DownloadManager {
    client: Client,
    progress_callback: Arc<Mutex<Box<dyn Fn(f32, &str, f64, Duration) + Send + Sync>>>,
    temp_dir: PathBuf,
}

impl DownloadManager {
    pub fn new<F>(progress_callback: F) -> Self 
    where 
        F: Fn(f32, &str, f64, Duration) + Send + Sync + 'static 
    {
        let client = Client::builder()
            .timeout(Duration::from_secs(1800))
            .build()
            .unwrap_or_default();
            
        let temp_dir = std::env::temp_dir().join("sinfar_installer");
        println!("Files will be downloaded to: {}", temp_dir.display());
        // Ensure temp directory exists
        if !temp_dir.exists() {
            if let Err(e) = std::fs::create_dir_all(&temp_dir) {
                warn!("Failed to create temp directory: {}", e);
            }
        }
        
        Self {
            client,
            progress_callback: Arc::new(Mutex::new(Box::new(progress_callback))),
            temp_dir,
        }
    }

    pub fn set_progress_callback<F>(&mut self, callback: F)
    where
        F: Fn(f32, &str, f64, Duration) + Send + Sync + 'static,
    {
        self.progress_callback = Arc::new(Mutex::new(Box::new(callback)));
    }
    
    /// Downloads a file with resume capability and progress tracking
    pub async fn download_file(&self, url: &str, output_path: &Path) -> Result<()> {
        info!("Starting download of {} to {}", url, output_path.display());
        
        // Create parent directories if they don't exist
        if let Some(parent) = output_path.parent() {
            if !parent.exists() {
                tokio::fs::create_dir_all(parent).await?;
            }
        }
        
        
        // Prepare headers for potential resume
        let mut headers = HeaderMap::new();
        headers.insert(USER_AGENT, HeaderValue::from_static("Sinfar NWN Installer"));
        // First make a HEAD request to get the total file size
        let response_head = self.client.get(url)
        .headers(headers)
        .send()
        .await
        .context("Failed to send HTTP request")?;
        
        let status = response_head.status();
            
        // Get content length for progress calculation
        let total_size = response_head.content_length().unwrap_or(0);
        let content_length = if status.as_u16() == 206 {
            if let Some(content_range) = response_head.headers().get("content-range") {
                let range_str = content_range.to_str().unwrap_or("");
                if let Some(size_part) = range_str.split('/').nth(1) {
                    size_part.parse::<u64>().unwrap_or(0)
                } else {
                    total_size
                }
            } else {
                total_size
            }
        } else {
            response_head.content_length().unwrap_or(total_size)
        };
        
        // Check if we have an existing file and its size
        let mut downloaded_size: u64 = 0;
        if output_path.exists() {
            let metadata = tokio::fs::metadata(output_path).await?;
            downloaded_size = metadata.len();

            // If the file size matches exactly, we're done
            if content_length > 0 && downloaded_size == content_length {
                info!("File is already completely downloaded: {} bytes", content_length);
                // Call progress callback with 100%
                let progress_callback = self.progress_callback.clone();
                let filename = output_path.file_name()
                    .and_then(|f| f.to_str())
                    .unwrap_or("file");
                let callback = progress_callback.lock().await;
                callback(1.0, filename, 0.0, Duration::from_secs(0));
                return Ok(());
            } else if downloaded_size > content_length {
                // If local file is larger than remote, start fresh
                info!("Local file size mismatch ({} > {}), starting fresh download", downloaded_size, content_length);
                downloaded_size = 0;
            }
        }
        
        // Prepare headers for potential resume
        let mut headers_toresume = HeaderMap::new();
        headers_toresume.insert(USER_AGENT, HeaderValue::from_static("Sinfar NWN Installer"));

        if downloaded_size > 0 {
            info!("Attempting to resume from byte {}", downloaded_size);
            headers_toresume.insert(RANGE, HeaderValue::from_str(&format!("bytes={}-", downloaded_size))?);
        }
        
        //Make the actual download request
        let response = self.client.get(url)
            .headers(headers_toresume)
            .send()
            .await
            .context("Failed to send HTTP request")?;
            
        let status = response.status();
        
        // Handle 416 Range Not Satisfiable specifically
        if status.as_u16() == 416 {
            // This can happen if the file is already complete
            if downloaded_size > 0 && downloaded_size == total_size {
                info!("File appears to be complete, got 416 with matching file size");
                let progress_callback = self.progress_callback.clone();
                let filename = output_path.file_name()
                    .and_then(|f| f.to_str())
                    .unwrap_or("file");
                let callback = progress_callback.lock().await;
                callback(1.0, filename, 0.0, Duration::from_secs(0));
                return Ok(());
            }
            // If sizes don't match, start fresh
            info!("Got 416 error but sizes don't match, starting fresh download");
            downloaded_size = 0;
            // Make a new request without range header
            let response = self.client.get(url)
                .header(USER_AGENT, "Sinfar NWN Installer")
                .send()
                .await
                .context("Failed to send HTTP request")?;
            if !response.status().is_success() {
                return Err(anyhow!("Server returned error status: {}", response.status()));
            }
        } else if !status.is_success() {
            return Err(anyhow!("Server returned error status: {}", status));
        }
        
        info!("Total download size: {} bytes", content_length);
        
        // Open file in appropriate mode
        let mut file = if downloaded_size > 0 {
            info!("Resuming download from byte {}", downloaded_size);
            tokio::fs::OpenOptions::new()
                .write(true)
                .append(true)
                .open(output_path)
                .await?
        } else {
            info!("Starting fresh download");
            tokio::fs::File::create(output_path).await?
        };
        
        // Stream the response
        let mut stream = response.bytes_stream();
        let progress_callback = self.progress_callback.clone();
        let filename = output_path.file_name()
            .and_then(|f| f.to_str())
            .unwrap_or("file");
            
        let mut last_update = std::time::Instant::now();
        let mut last_downloaded_size = downloaded_size;
        let start_time = std::time::Instant::now();
        
        while let Some(chunk_result) = stream.next().await {
            let chunk = chunk_result?;
            file.write_all(&chunk).await?;
            
            downloaded_size += chunk.len() as u64;
            
            // Only update progress every 100ms to avoid overwhelming the UI
            let now = std::time::Instant::now();
            if now.duration_since(last_update) > Duration::from_millis(100) || content_length == downloaded_size {
                if content_length > 0 {
                    let progress = downloaded_size as f32 / content_length as f32;
                    
                    // Calculate speed in bytes per second
                    let elapsed = now.duration_since(last_update).as_secs_f64();
                    let bytes_since_last = downloaded_size - last_downloaded_size;
                    let speed = bytes_since_last as f64 / elapsed;
                    
                    // Calculate estimated time remaining
                    let remaining_bytes = content_length - downloaded_size;
                    let estimated_remaining = if speed > 0.0 {
                        Duration::from_secs_f64(remaining_bytes as f64 / speed)
                    } else {
                        Duration::from_secs(0)
                    };

                    let callback = progress_callback.lock().await;
                    callback(progress, filename, speed, estimated_remaining);
                }
                last_update = now;
                last_downloaded_size = downloaded_size;
            }
        }
        
        info!("Download completed: {}", output_path.display());
        Ok(())
    }
    
    /// Downloads all required files for the installation
    pub async fn download_all_files(&self, game_type: &GameType) -> Result<Vec<PathBuf>> {
        let mut downloaded_files = Vec::new();
        
        // Always download the content files
        let content_zip_path = self.temp_dir.join("sinfar_all_files_v30.7z");
        
        // Always try to download/resume the content files - don't skip based on existing size
        self.download_file(
            "https://sinfar.net/haks/sinfar_all_files_v30.7z", 
            &content_zip_path
        ).await?;
        
        downloaded_files.push(content_zip_path);
        
        // Download version-specific files
        if *game_type == GameType::Diamond {
            let launcher_path = self.temp_dir.join("sinfarx.exe");
            
            // Always attempt download/resume
            self.download_file(
                "https://nwn.sinfar.net/files/sinfarx.exe", 
                &launcher_path
            ).await?;
            
            downloaded_files.push(launcher_path);
        } else if *game_type == GameType::EnhancedEdition {
            // For EE, download the appropriate launcher for the platform
            #[cfg(windows)]
            {
                let launcher_zip_path = self.temp_dir.join("sinfarLauncher.zip");
                
                // Always attempt download/resume
                self.download_file(
                    "https://nwn.sinfar.net/files/sinfarx/8181/win32_8181.zip", 
                    &launcher_zip_path
                ).await?;
                
                downloaded_files.push(launcher_zip_path);
            }
            
            #[cfg(unix)]
            {
                let launcher_zip_path = self.temp_dir.join("sinfarLauncher.zip");
                
                // Always attempt download/resume
                self.download_file(
                    "https://nwn.sinfar.net/files/sinfarx/8181/linux_8181.zip", 
                    &launcher_zip_path
                ).await?;
                
                downloaded_files.push(launcher_zip_path);
            }
        }
        
        Ok(downloaded_files)
    }
    
    /// Check if a file exists and meets a minimum size requirement
    fn file_exists_with_size(&self, path: &Path, min_size: u64) -> bool {
        if let Ok(meta) = metadata(path) {
            meta.is_file() && meta.len() >= min_size
        } else {
            false
        }
    }
    
    /// Get the path to the temporary directory
    pub fn get_temp_dir(&self) -> &Path {
        &self.temp_dir
    }
}