// src/installer/extractor.rs
use anyhow::{Result, Context};
use sevenz_rust::decompress;
use std::path::{Path, PathBuf};
use std::sync::Arc;
use tokio::sync::Mutex;
use log::{info, error};
use std::fs;
use std::io::{Read, Seek};
use crate::app::GameType;

pub struct ExtractionManager {
    progress_callback: Arc<Mutex<Box<dyn Fn(f32, &str, std::time::Duration) + Send + Sync>>>,
    last_update: Arc<Mutex<std::time::Instant>>,
    last_processed: Arc<Mutex<u64>>,
}

impl ExtractionManager {
    pub fn new<F>(progress_callback: F) -> Self 
    where 
        F: Fn(f32, &str, std::time::Duration) + Send + Sync + 'static 
    {
        Self {
            progress_callback: Arc::new(Mutex::new(Box::new(progress_callback))),
            last_update: Arc::new(Mutex::new(std::time::Instant::now())),
            last_processed: Arc::new(Mutex::new(0)),
        }
    }
    
    // Helper function to calculate estimated time remaining
    async fn update_progress(&self, processed: u64, total: u64, filename: &str) {
        let progress = processed as f32 / total as f32;
        let now = std::time::Instant::now();
        
        let mut last_update = self.last_update.lock().await;
        let mut last_processed = self.last_processed.lock().await;
        
        let elapsed = now.duration_since(*last_update).as_secs_f64();
        let bytes_since_last = processed - *last_processed;
        let speed = bytes_since_last as f64 / elapsed;
        
        let remaining_bytes = total - processed;
        let estimated_remaining = if speed > 0.0 {
            std::time::Duration::from_secs_f64(remaining_bytes as f64 / speed)
        } else {
            std::time::Duration::from_secs(0)
        };
        
        *last_update = now;
        *last_processed = processed;
        
        let callback = self.progress_callback.lock().await;
        callback(progress, filename, estimated_remaining);
    }

    /// Extract a 7z archive with progress reporting
    pub async fn extract_7z(&self, archive_path: &Path, output_dir: &Path) -> Result<()> {
        info!("Starting extraction of 7z archive");
        info!("Archive path: {}", archive_path.display());
        info!("Output directory: {}", output_dir.display());
        
        // Check if archive exists
        if !archive_path.exists() {
            error!("Archive file does not exist: {}", archive_path.display());
            return Err(anyhow::anyhow!("Archive file does not exist"));
        }
        
        // Validate 7z signature and get archive size
        let mut file = fs::File::open(&archive_path)?;
        let mut signature = [0u8; 6];
        file.read_exact(&mut signature)?;
        
        // 7z files start with bytes [0x37, 0x7A, 0xBC, 0xAF, 0x27, 0x1C]
        if signature != [0x37, 0x7A, 0xBC, 0xAF, 0x27, 0x1C] {
            error!("Invalid 7z file signature: {:?}", signature);
            return Err(anyhow::anyhow!("Invalid 7z file - got wrong signature. The downloaded file might be corrupted or might be an error page"));
        }
        info!("7z signature validation passed");
        
        // Make sure the output directory exists
        if !output_dir.exists() {
            info!("Creating output directory: {}", output_dir.display());
            tokio::fs::create_dir_all(output_dir).await?;
        }
        
        // Get file size for progress calculation
        let archive_size = fs::metadata(archive_path)?.len();
        info!("Archive size: {} bytes", archive_size);
        
        // We'll handle extraction in a blocking task since sevenz-rust is synchronous
        let archive_path = archive_path.to_path_buf();
        let output_dir = output_dir.to_path_buf();
        let progress_callback = self.progress_callback.clone();
        let last_update = self.last_update.clone();
        let last_processed = self.last_processed.clone();
        
        info!("Starting blocking extraction task");
        tokio::task::spawn_blocking(move || -> Result<()> {
            // Open the archive for processing
            let archive_file = fs::File::open(&archive_path)?;
            
            // Create a wrapper around the file that tracks read progress
            struct ProgressReader<R: Read + Seek> {
                inner: R,
                bytes_read: u64,
                total_size: u64,
                last_progress: f32,
                progress_callback: Arc<Mutex<Box<dyn Fn(f32, &str, std::time::Duration) + Send + Sync>>>,
                last_update: Arc<Mutex<std::time::Instant>>,
                last_processed: Arc<Mutex<u64>>,
            }
            
            impl<R: Read + Seek> Read for ProgressReader<R> {
                fn read(&mut self, buf: &mut [u8]) -> std::io::Result<usize> {
                    let bytes = self.inner.read(buf)?;
                    self.bytes_read += bytes as u64;
                    
                    // Calculate progress as a percentage
                    let progress = self.bytes_read as f32 / self.total_size as f32;
                    
                    // Only update progress if it has changed by at least 1%
                    if progress - self.last_progress >= 0.01 {
                        self.last_progress = progress;
                        let progress_callback_clone = self.progress_callback.clone();
                        let last_update_clone = self.last_update.clone();
                        let last_processed_clone = self.last_processed.clone();
                        tokio::runtime::Handle::current().block_on(async move {
                            let callback = progress_callback_clone.lock().await;
                            let now = std::time::Instant::now();
                            let mut last_update = last_update_clone.lock().await;
                            let mut last_processed = last_processed_clone.lock().await;
                            
                            let elapsed = now.duration_since(*last_update).as_secs_f64();
                            let bytes_since_last = self.bytes_read - *last_processed;
                            let speed = bytes_since_last as f64 / elapsed;
                            
                            let remaining_bytes = self.total_size - self.bytes_read;
                            let estimated_remaining = if speed > 0.0 {
                                std::time::Duration::from_secs_f64(remaining_bytes as f64 / speed)
                            } else {
                                std::time::Duration::from_secs(0)
                            };
                            
                            *last_update = now;
                            *last_processed = self.bytes_read;
                            
                            callback(progress, "Extracting...", estimated_remaining);
                        });
                    }
                    
                    Ok(bytes)
                }
            }
            
            impl<R: Read + Seek> Seek for ProgressReader<R> {
                fn seek(&mut self, pos: std::io::SeekFrom) -> std::io::Result<u64> {
                    self.inner.seek(pos)
                }
            }
            
            let progress_reader = ProgressReader {
                inner: archive_file,
                bytes_read: 0,
                total_size: archive_size,
                last_progress: 0.0,
                progress_callback: progress_callback.clone(),
                last_update: last_update.clone(),
                last_processed: last_processed.clone(),
            };
            
            info!("Starting decompression to: {}", output_dir.display());
            
            // Decompress using our progress-tracking reader
            match decompress(progress_reader, &output_dir) {
                Ok(_) => info!("Decompression completed successfully"),
                Err(e) => {
                    error!("Decompression failed: {}", e);
                    return Err(anyhow::anyhow!("Decompression failed: {}", e));
                }
            }
            
            // Call progress update with 100% after decompression
            let progress_callback_clone = progress_callback.clone();
            tokio::runtime::Handle::current().block_on(async move {
                let callback = progress_callback_clone.lock().await;
                callback(1.0, "Complete", std::time::Duration::from_secs(0));
            });
            
            info!("Extraction task completed successfully");
            Ok(())
        }).await??;
        
        info!("7z extraction process completed");
        Ok(())
    }
    
    /// Extract a ZIP archive with progress reporting
    pub async fn extract_zip(&self, archive_path: &Path, output_dir: &Path) -> Result<()> {
        info!("Extracting ZIP archive: {} -> {}", archive_path.display(), output_dir.display());
        
        // Make sure the output directory exists
        if !output_dir.exists() {
            tokio::fs::create_dir_all(output_dir).await?;
        }
        
        // We'll handle extraction in a blocking task since the zip crate is synchronous
        let archive_path = archive_path.to_path_buf();
        let output_dir = output_dir.to_path_buf();
        let progress_callback = self.progress_callback.clone();
        let last_update = self.last_update.clone();
        let last_processed = self.last_processed.clone();
        
        tokio::task::spawn_blocking(move || -> Result<()> {
            let file = fs::File::open(&archive_path)?;
            let mut archive = zip::ZipArchive::new(file)?;
            
            // Calculate total uncompressed size
            let total_size: u64 = {
                let mut size = 0;
                for i in 0..archive.len() {
                    if let Ok(file) = archive.by_index(i) {
                        size += file.size();
                    }
                }
                size
            };
            
            info!("ZIP archive total uncompressed size: {} bytes", total_size);
            
            let mut processed_bytes: u64 = 0;
            
            // Process files in batches for better performance
            let mut pending_dirs = Vec::new();
            
            // First pass: create all directories (this prevents race conditions)
            for i in 0..archive.len() {
                let file = archive.by_index(i)?;
                let outpath = match file.enclosed_name() {
                    Some(path) => output_dir.join(path),
                    None => continue,
                };
                
                if file.name().ends_with('/') {
                    pending_dirs.push(outpath);
                } else if let Some(p) = outpath.parent() {
                    pending_dirs.push(p.to_path_buf());
                }
            }
            
            // Create all directories at once
            for dir in pending_dirs.iter().collect::<std::collections::HashSet<_>>() {
                fs::create_dir_all(dir)?;
            }
            
            // Second pass: extract all files
            for i in 0..archive.len() {
                let mut file = archive.by_index(i)?;
                let outpath = match file.enclosed_name() {
                    Some(path) => output_dir.join(path),
                    None => continue,
                };
                
                if !file.name().ends_with('/') {
                    let mut outfile = fs::File::create(&outpath)?;
                    let file_size = file.size();
                    std::io::copy(&mut file, &mut outfile)?;
                    processed_bytes += file_size;
                    
                    // Update progress based on processed bytes
                    let progress = processed_bytes as f32 / total_size as f32;
                    let filename = outpath.file_name()
                        .and_then(|f| f.to_str())
                        .unwrap_or("unknown");
                    
                    // Update progress less frequently to reduce overhead
                    if i % 10 == 0 || progress >= 1.0 {
                        let progress_callback_clone = progress_callback.clone();
                        let last_update_clone = last_update.clone();
                        let last_processed_clone = last_processed.clone();
                        tokio::runtime::Handle::current().block_on(async move {
                            let callback = progress_callback_clone.lock().await;
                            let now = std::time::Instant::now();
                            let mut last_update = last_update_clone.lock().await;
                            let mut last_processed = last_processed_clone.lock().await;
                            
                            let elapsed = now.duration_since(*last_update).as_secs_f64();
                            let bytes_since_last = processed_bytes - *last_processed;
                            let speed = bytes_since_last as f64 / elapsed;
                            
                            let remaining_bytes = total_size - processed_bytes;
                            let estimated_remaining = if speed > 0.0 {
                                std::time::Duration::from_secs_f64(remaining_bytes as f64 / speed)
                            } else {
                                std::time::Duration::from_secs(0)
                            };
                            
                            *last_update = now;
                            *last_processed = processed_bytes;
                            
                            callback(progress, filename, estimated_remaining);
                        });
                    }
                }
                
                // Fix file permissions on Unix
                #[cfg(unix)]
                {
                    use std::os::unix::fs::PermissionsExt;
                    if let Some(mode) = file.unix_mode() {
                        fs::set_permissions(&outpath, fs::Permissions::from_mode(mode))?;
                    }
                }
            }
            
            // Ensure we show 100% at the end
            let progress_callback_clone = progress_callback.clone();
            tokio::runtime::Handle::current().block_on(async move {
                let callback = progress_callback_clone.lock().await;
                callback(1.0, "Complete", std::time::Duration::from_secs(0));
            });
            
            info!("ZIP extraction completed successfully");
            Ok(())
        }).await??;
        
        Ok(())
    }
    
    /// Handles extraction of all downloaded files to their appropriate locations  
    pub async fn install_all_files(
        &self, 
        downloaded_files: &[PathBuf], 
        install_path: &Path, 
        game_type: &GameType, 
        ee_exe_path: Option<&Path>
    ) -> Result<()> {
        info!("Starting installation of all files");
        info!("Install path: {}", install_path.display());
        info!("Game type: {:?}", game_type);
        if let Some(ee_path) = ee_exe_path {
            info!("EE executable path: {}", ee_path.display());
        }

        info!("Available downloaded files:");
        for file in downloaded_files {
            info!("  {}", file.display());
        }

        // Find and extract the main content files
        let content_archive = downloaded_files.iter()
            .find(|p| p.file_name().map_or(false, |f| f.to_str().unwrap_or("").contains("sinfar_all_files")))
            .context("Content archive not found in downloaded files")?;
        
        info!("Found content archive: {}", content_archive.display());
            
        // Extract main content
        info!("Starting extraction of main content");
        self.extract_7z(content_archive, install_path).await?;
        
        // Handle game-specific files
        info!("Processing game-specific files");
        match game_type {
            GameType::Diamond => {
                info!("Installing Diamond edition launcher");
                // For Diamond, find and copy the launcher
                let launcher = downloaded_files.iter()
                    .find(|p| p.file_name().map_or(false, |f| f.to_str().unwrap_or("").contains("sinfarx.exe")))
                    .context("Launcher executable not found in downloaded files")?;
                    
                // Copy launcher to install directory
                let target_path = install_path.join("sinfarx.exe");
                info!("Copying launcher from {} to {}", launcher.display(), target_path.display());
                tokio::fs::copy(launcher, &target_path).await?;
                info!("Launcher copied successfully");
            }
            GameType::EnhancedEdition => {
                info!("Installing Enhanced Edition launcher");
                // For EE, find the launcher zip
                let launcher_zip = downloaded_files.iter()
                    .find(|p| p.file_name().map_or(false, |f| f.to_str().unwrap_or("").contains("sinfarLauncher.zip")))
                    .context("Launcher ZIP not found in downloaded files")?;
                    
                // Extract to the EE executable directory
                if let Some(ee_path) = ee_exe_path {
                    // Create the bin/win32_8181 directory if it doesn't exist (Windows)
                    // or bin/linux_8181 directory (Linux)
                    #[cfg(windows)]
                    let extract_dir = ee_path.join("bin");
                    
                    #[cfg(unix)]
                    let extract_dir = ee_path.join("bin");
                    
                    info!("Extracting EE launcher to {}", extract_dir.display());
                    // Extract the launcher
                    self.extract_zip(launcher_zip, &extract_dir).await?;
                    info!("EE launcher extracted successfully");
                } else {
                    error!("EE executable path is required but not provided");
                    return Err(anyhow::anyhow!("EE executable path is required for EE installation"));
                }
            }
        }
        
        info!("All files installed successfully");
        Ok(())
    }
}