// src/util/platform.rs
use std::path::{Path, PathBuf};
use anyhow::Result;

/// Platform-agnostic function to get the user's documents directory
pub fn get_documents_dir() -> Option<PathBuf> {
    dirs::document_dir()
}

/// Get the default installation directory for NWN based on platform
pub fn get_default_nwn_dir(is_ee: bool) -> PathBuf {
    if is_ee {
        #[cfg(windows)]
        {
            if let Some(docs) = get_documents_dir() {
                return docs.join("Neverwinter Nights");
            } else {
                return PathBuf::from("C:\\Users\\Public\\Documents\\Neverwinter Nights");
            }
        }
        
        #[cfg(unix)]
        {
            if let Some(home) = dirs::home_dir() {
                return home.join(".local/share/Neverwinter Nights");
            } else {
                return PathBuf::from("/home").join(whoami::username()).join(".local/share/Neverwinter Nights");
            }
        }
    } else {
        // Diamond Edition
        #[cfg(windows)]
        {
            // Try common installation paths for GOG and Steam
            let gog_path = PathBuf::from("C:\\Program Files (x86)\\GOG Galaxy\\Games\\NWN Diamond");
            if gog_path.exists() {
                return gog_path;
            }
            
            let steam_path = PathBuf::from("C:\\Program Files (x86)\\Steam\\steamapps\\common\\NWN Diamond");
            if steam_path.exists() {
                return steam_path;
            }
            
            // Default to GOG path even if it doesn't exist
            return gog_path;
        }
        
        #[cfg(unix)]
        {
            if let Some(home) = dirs::home_dir() {
                // Check common Linux installation locations
                let steam_path = home.join(".steam/steam/steamapps/common/NWN Diamond");
                if steam_path.exists() {
                    return steam_path;
                }
                
                let gog_path = home.join("GOG Games/NWN Diamond");
                if gog_path.exists() {
                    return gog_path;
                }
                
                return home.join(".local/share/NWN Diamond");
            } else {
                return PathBuf::from("/home").join(whoami::username()).join(".local/share/NWN Diamond");
            }
        }
    }
}

/// Get the executable name with the correct extension based on platform
pub fn get_executable_name(base_name: &str) -> String {
    #[cfg(windows)]
    return format!("{}.exe", base_name);
    
    #[cfg(unix)]
    return base_name.to_string();
}

/// Check if we need elevated permissions for a path
pub fn needs_elevation(path: &Path) -> bool {
    #[cfg(windows)]
    {
        // Common paths that typically need elevation on Windows
        path.to_string_lossy().to_lowercase().contains("program files") || 
        path.to_string_lossy().to_lowercase().contains("programfiles") ||
        path.to_string_lossy().to_lowercase().contains("\\windows\\")
    }
    
    #[cfg(unix)]
    {
        // Common paths that typically need elevation on Linux
        path.to_string_lossy().starts_with("/usr") || 
        path.to_string_lossy().starts_with("/opt") ||
        path.to_string_lossy().starts_with("/bin") ||
        path.to_string_lossy().starts_with("/sbin")
    }
}

/// Request elevated permissions
pub fn request_elevation() -> Result<bool> {
    #[cfg(windows)]
    {
        use std::process::Command;
        use std::env;
        
        let executable = env::current_exe()?;
        
        // Don't try to re-elevate if we're already elevated
        if is_elevated()? {
            return Ok(true);
        }
        
        // Use the Windows ShellExecute to prompt for elevation
        let result = Command::new("powershell")
            .args(&[
                "-Command",
                &format!("Start-Process -FilePath '{}' -Verb RunAs", executable.to_string_lossy()),
            ])
            .output()?;
            
        // If the command executed successfully, exit the current non-elevated process
        if result.status.success() {
            std::process::exit(0);
        }
        
        Ok(false)
    }
    
    #[cfg(unix)]
    {
        use std::process::Command;
        use std::env;
        
        let executable = env::current_exe()?;
        
        // Don't try to re-elevate if we're already elevated
        if is_elevated()? {
            return Ok(true);
        }
        
        // Try pkexec first (GNOME/KDE)
        if Command::new("which").arg("pkexec").output()?.status.success() {
            let _ = Command::new("pkexec")
                .arg(executable)
                .spawn()?;
            std::process::exit(0);
        }
        
        // Try gksudo next (older GNOME)
        if Command::new("which").arg("gksudo").output()?.status.success() {
            let _ = Command::new("gksudo")
                .arg("--")
                .arg(executable)
                .spawn()?;
            std::process::exit(0);
        }
        
        // Try kdesudo (KDE)
        if Command::new("which").arg("kdesudo").output()?.status.success() {
            let _ = Command::new("kdesudo")
                .arg("--")
                .arg(executable)
                .spawn()?;
            std::process::exit(0);
        }
        
        // Fallback to terminal-based sudo
        if Command::new("which").arg("sudo").output()?.status.success() {
            let _ = Command::new("sudo")
                .arg(executable)
                .spawn()?;
            std::process::exit(0);
        }
        
        Ok(false)
    }
}

/// Check if the current process has elevated permissions
pub fn is_elevated() -> Result<bool> {
    #[cfg(windows)]
    {
        use std::process::Command;
        
        // This command succeeds only if the process has admin privileges
        let output = Command::new("net")
            .args(&["session"])
            .output()?;
            
        Ok(output.status.success())
    }
    
    #[cfg(unix)]
    {
        // On Unix-like systems, check if we're root (uid 0)
        Ok(unsafe { libc::geteuid() == 0 })
    }
}

/// Create a desktop shortcut in a platform-specific way
pub fn create_desktop_shortcut(
    target: &Path, 
    name: &str, 
    icon: Option<&Path>,
    description: Option<&str>
) -> Result<()> {
    #[cfg(windows)]
    {
        use std::process::Command;
        
        let desktop = if let Some(desktop_dir) = dirs::desktop_dir() {
            desktop_dir
        } else {
            return Err(anyhow::anyhow!("Could not find desktop directory"));
        };
        
        let shortcut_path = desktop.join(format!("{}.lnk", name));
        
        // Create Windows shortcut using PowerShell
        let mut ps_cmd = String::from("$ws = New-Object -ComObject WScript.Shell; ");
        ps_cmd.push_str(&format!("$s = $ws.CreateShortcut('{}'); ", shortcut_path.to_string_lossy()));
        ps_cmd.push_str(&format!("$s.TargetPath = '{}'; ", target.to_string_lossy()));
        
        if let Some(icon_path) = icon {
            ps_cmd.push_str(&format!("$s.IconLocation = '{}'; ", icon_path.to_string_lossy()));
        }
        
        if let Some(desc) = description {
            ps_cmd.push_str(&format!("$s.Description = '{}'; ", desc));
        }
        
        ps_cmd.push_str("$s.Save()");
        
        Command::new("powershell")
            .args(&["-Command", &ps_cmd])
            .output()?;
            
        Ok(())
    }
    
    #[cfg(unix)]
    {
        use std::fs::File;
        use std::io::Write;
        
        let desktop = if let Some(desktop_dir) = dirs::desktop_dir() {
            desktop_dir
        } else {
            return Err(anyhow::anyhow!("Could not find desktop directory"));
        };
        
        let shortcut_path = desktop.join(format!("{}.desktop", name));
        
        let mut file = File::create(&shortcut_path)?;
        
        writeln!(file, "[Desktop Entry]")?;
        writeln!(file, "Type=Application")?;
        writeln!(file, "Name={}", name)?;
        writeln!(file, "Exec={}", target.to_string_lossy())?;
        
        if let Some(icon_path) = icon {
            writeln!(file, "Icon={}", icon_path.to_string_lossy())?;
        }
        
        if let Some(desc) = description {
            writeln!(file, "Comment={}", desc)?;
        }
        
        writeln!(file, "Terminal=false")?;
        
        // Make executable
        use std::os::unix::fs::PermissionsExt;
        let mut perms = std::fs::metadata(&shortcut_path)?.permissions();
        perms.set_mode(0o755);  // rwxr-xr-x
        std::fs::set_permissions(&shortcut_path, perms)?;
        
        Ok(())
    }
}